package br.usp.ime.pictware.business.util.dao;

import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import br.usp.ime.pictware.business.util.dao.exception.DAOException;

/**
 * @author goliva
 */
public final class DAOFactory {

    /**
     * Log.
     */
    private static final Logger LOG = Logger.getLogger(DAOFactory.class.getName());

    /**
     * Instncia da DAOFactory.
     */
    private static DAOFactory instance = new DAOFactory();

    /**
     * Construtor padro.
     */
    private DAOFactory() {
    }
  
     /**
     * Recupera o DAO configurado no arquivo dao.properties. 
     *  
     * @param daoClass Classe do Dao
     * @return Uma instancia do dao
     * @throws DAOException exceo lanada quando no for possvel obter o Dao
     * 
     */
    @SuppressWarnings("unchecked")
    public <T extends DAO> T getDao(Class<T> daoClass) throws DAOException {
        final Properties daoProperties = this.getProperties();
        final String concreteClass =
                daoProperties.getProperty(daoClass.getName());
        
        // Verificando se o dao foi configurado corretamente
        if (concreteClass == null)
            throw new DAOException("Erro recuperando classe concreta para a interface: " + daoClass.getName());
                
        try {
            final Class clazz = Class.forName(concreteClass);
            return (T) clazz.newInstance();
        } catch (ClassNotFoundException e) {
            LOG.log(Level.SEVERE, "Classe concreta no encontrada para o dao: " + daoClass.getName(), e);
            throw new DAOException(e);
        } catch (IllegalAccessException e) {
            LOG.log(Level.SEVERE, "Erro criando classe concreta para o dao: " + daoClass.getName(), e);
            throw new DAOException(e);
        } catch (InstantiationException e) {
            LOG.log(Level.SEVERE, "Erro criando classe concreta para o dao: " + daoClass.getName(), e);
            throw new DAOException(e);
        }
        
    }

    /**
     * Obtm o arquivo properties do DAO.
     * @return properties do DAO.
     * @throws DAOException se no conseguir ler o arquivo.
     */
    private Properties getProperties() throws DAOException {
        final Properties properties = new Properties();

        try {
        	properties.load(Thread.currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream("dao.properties"));
        } catch (IOException e) {
            LOG.log(Level.SEVERE, e.getMessage(), e);
            throw new DAOException(e);
        }

        return properties;
    }
	
    public static DAOFactory getInstance() {
        return instance;
    }

}
